# Extensions (v5.0.0)

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->

- [Overview](#extensions-v500)
  - [What is an Extension?](#what-is-an-extension)
      - [Examples of functionality that are implemented via built-in extensions:](#examples-of-functionality-that-are-implemented-via-built-in-extensions)
      - [Examples of custom built external extensions:](#examples-of-custom-built-external-extensions)
  - [What is a Button?](#what-is-a-button)
      - [All of the built-in MediumEditor buttons are just Button Extensions with different [configuration](https://github.com/yabwe/medium-editor/blob/master/src/js/defaults/buttons.js):](#all-of-the-built-in-mediumeditor-buttons-are-just-button-extensions-with-different-configurationhttpsgithubcomyabwemedium-editorblobmastersrcjsdefaultsbuttonsjs)
      - [Examples of custom built external buttons:](#examples-of-custom-built-external-buttons)
  - [What is a Form Extension?](#what-is-a-form-extension)
      - [Built-in Form Extensions](#built-in-form-extensions)
- [Extensions](#extensions)
  - [Example: DisableContextMenuExtension](#example-disablecontextmenuextension)
    - [Define the Extension](#define-the-extension)
    - [Attaching To Context Menu Event](#attaching-to-context-menu-event)
    - [Adding Functionality](#adding-functionality)
    - [Leveraging Custom Event Listeners](#leveraging-custom-event-listeners)
  - [Extension Interface](#extension-interface)
    - [`name` _(string)_](#name-_string_)
    - [`init()`](#init)
    - [`checkState(node)`](#checkstatenode)
    - [`destroy()`](#destroy)
    - [`queryCommandState()`](#querycommandstate)
    - [`isActive()`](#isactive)
    - [`isAlreadyApplied(node)`](#isalreadyappliednode)
    - [`setActive()`](#setactive)
    - [`setInactive()`](#setinactive)
  - [Extension Helpers](#extension-helpers)
    - [`base` _(MediumEditor)_](#base-_mediumeditor_)
    - [`window` _(Window)_](#window-_window_)
    - [`document` _(Document)_](#document-_document_)
    - [`getEditorElements()`](#geteditorelements)
    - [`getEditorId()`](#geteditorid)
    - [`getEditorOption(option)`](#geteditoroptionoption)
  - [Extension Proxy Methods](#extension-proxy-methods)
    - [`execAction(action, opts)`](#execactionaction-opts)
    - [`on(target, event, listener, useCapture)`](#ontarget-event-listener-usecapture)
    - [`off(target, event, listener, useCapture)`](#offtarget-event-listener-usecapture)
    - [`subscribe(name, listener)`](#subscribename-listener)
    - [`trigger(name, data, editable)`](#triggername-data-editable)
- [Buttons](#buttons)
  - [Example: HighlighterButton](#example-highlighterbutton)
  - [Button Extension API](#button-extension-api)
- [Form Buttons](#form-buttons)
  - [Example: Anchor](#example-anchor)
  - [From Extension API](#from-extension-api)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## What is an Extension?

**Extensions** are custom actions or commands that can be passed in via the `extensions` option to medium-editor.  They can replace existing buttons if they share the same name, or can add additional custom functionality into the editor.  Extensions can be implemented in any way, and just provide a way to hook into medium-editor.  New extensions can be created by extending the exposed `MediumEditor.Extension` object via `MediumEditor.Extension.extend()`.


#### Examples of functionality that are implemented via built-in extensions:
* [The MediumEditor Toolbar](https://github.com/yabwe/medium-editor/blob/toolbar-extension/src/js/extensions/toolbar.js)
  * The entire toolbar which contains all of the MediumEditor buttons is implemented as an extension!
* [Auto-Link Detection](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/auto-link.js)
  * Automatically detecting when a url has been added and converting it into an anchor tag
* [Anchor Preview Tooltip](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/anchor-preview.js)
  * When a user hovers over a link a tooltip is displayed showing the href of the anchor tag
* [Image Drag & Drop](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/image-dragging.js)
  * Allows users to drag and drop images into the editor
* [Keyboard Commands](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/keyboard-commands.js)
  * Mapping keyboard shortcuts to various commands
* [Placeholder Text](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/placeholder.js)
  * Shows placeholder text when the editor is empty
* [Paste Handling](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/paste.js)
  * Handles filtering and modifying text that is pasted into the editor

#### Examples of custom built external extensions:
* [MediumEditor markdown](https://github.com/IonicaBizau/medium-editor-markdown)
  * A Medium Editor extension to add markdown support.
* [MediumEditor Insert Plugin](https://github.com/orthes/medium-editor-insert-plugin)
  * Enables users to insert into the editor various types of content like images or embeds.


## What is a Button?
**Buttons** are a specific type of Extension which have a contract with the MediumEditor toolbar.  Buttons have specific lifecycle methods that MediumEditor and the toolbar use to interact with these specific types of Extensions.  These contract create easy hooks, allowing custom buttons to:
* Display an element in the toolbar _(ie a clickable button/link)_
* Execute an action on the editor text when clicked _(ie bold, underline, blockquote, etc.)_
* Update the appearance of the element based on the user selection _(ie the bold button looks 'active' if the selected text is already bold, 'inactive' if the text is not bold)_

#### All of the built-in MediumEditor buttons are just Button Extensions with different [configuration](https://github.com/yabwe/medium-editor/blob/master/src/js/defaults/buttons.js):
* bold, italic, underline, strikethrough
* subscript, superscript
* image
* quote, pre
* orderedlist, unorderedlist
* indent, outdent
* justifyLeft, justifyCenter, justifyRight, justifyFull
* h1, h2, h3, h4, h5, h6
* removeFormat

#### Examples of custom built external buttons:
* [MediumEditor tables](https://github.com/yabwe/medium-editor-tables)
  * Extension to add a table button/behavior to MediumEditor
* [MediumEditor Custom HTML](https://github.com/jillix/medium-editor-custom-html)
  * An extension that inserts custom HTML using a new button in the Medium Editor toolbar
* [MediumButton](https://github.com/arcs-/MediumButton)
  * Extends your Medium Editor with the possibility add buttons.


## What is a Form Extension?
**Form Extensions** are a specific type of Button Extension which collect input from the user via the toolbar.  Form Extensions extend from Button, and thus inherit all of the lifecycle methods of a Button.  In addition, Form Extensions have some additional methods exposed to interact with MediumEditor and provide some common functionality.

#### Built-in Form Extensions
* [Anchor Button](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/anchor.js)
  * The `'anchor'` Button is actually a form extension, which when clicked, prompts the the user for a url (as well as some optional checkboxes) via a control in the toolbar and converts the selected text into a link.  If the selection is already a link, clicking the button unwraps text within the anchor tag.
* [FontSize Button (beta)](https://github.com/yabwe/medium-editor/blob/master/src/js/extensions/fontsize.js)
  * The `'fontsize'` Button is a form extension, which when clicked, allows the user to modify the size of the existing text via a control in the toolbar.


# Extensions

## Example: DisableContextMenuExtension

You can find a demo of this example in the source code via [extension-example.html](https://github.com/yabwe/medium-editor/blob/master/demo/extension-example.html).

To interact with the demo, load the page from your fork in a browser via: 

`file://[Medium Editor Source Root]/demo/extension-example.html`

### Define the Extension

As a simple example, let's create an extension that disables the context menu from appearing when the user right-clicks on the editor.

Defining this extension is as simple as calling `MediumEditor.Extension.extend()` and passing in the methods/properties we want to override.

```js
var DisableContextMenuExtension = MediumEditor.Extension.extend({
  name: 'disable-context-menu'
});
```

We now have an extension named `'disable-context-menu'` which we can pass into MediumEditor like this:

```js
var editor = new MediumEditor('.editable', {
  extensions: {
    'disable-context-menu': new DisableContextMenuExtension()
  }
});
```

### Attaching To Context Menu Event

To make the extension actually do something, we'll want to attach to the `contextmenu` event on all **elements** of the editor.  We can set this up by implementing the `init()` method, which is called on every Extension during setup of MediumEditor:

```js
var DisableContextMenuExtension = MediumEditor.Extension.extend({
  name: 'disable-context-menu',

  init: function () {
    this.getEditorElements().forEach(function (element) {
      this.base.on(element, 'contextmenu', this.handleContextmenu.bind(this));
    }, this);
  },

  handleContextmenu: function (event) { }
});
```

Here, we're leveraging some of the helpers that are available to all Extensions.

* We're using `this.getEditorElements()`, which is a helper function to give us an array containing all **elements** maintained by this editor.
* We're using `this.base`, which is a reference to the MediumEditor instance.
* We're using `this.base.on()`, which is a [method of MediumEditor](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#ontarget-event-listener-usecapture) for attaching to DOM Events. Using this method ensures our event handlers will be detached when MediumEditor is destroyed.

**NOTE**

* There are a few helper methods that allow us to make calls directly into the MediumEditor instance without having to reference `this.base`. One of them is a reference to the `on()` method, so instead of the above code we can just use `this.on(element, 'contextmenu', this.handleContextmenu.bind(this))` which is what we'll use in the rest of the example.

### Adding Functionality

So, the last piece we need is to handle the `contextmenu` event and prevent the default action:

```js
var DisableContextMenuExtension = MediumEditor.Extension.extend({
  name: 'disable-context-menu',

  init: function () {
    this.getEditorElements().forEach(function (element) {
      this.base.on(element, 'contextmenu', this.handleContextmenu.bind(this));
    }, this);
  },

  handleContextmenu: function (event) {
    event.preventDefault();
  }
});
```

Now we have a working extension which prevents the context menu from showing up for any of the **elements**.  Let's add some more functionality to allow for toggling this feature on and off.

### Leveraging Custom Event Listeners

Let's say we wanted to support toggling on/off the disable-context-menu extension, for a specific **element**, whenever the user presses ESCAPE.  To do this, we'll need to add 2 pieces of functionality:

1. Listen to the `keydown` event on each **element**. For this, we can leverage the built-in [`editableKeyDown` custom event](https://github.com/yabwe/medium-editor/wiki/Custom-Events#editablekeydown).  This allows us to use the 2nd argument of custom event listeners (the active editor **element**) to toggle on/off a `data-allow-context-menu` attribute on the **element**.

2. When the `contextmenu` event fires, we only want to prevent the context menu from appearing if the `data-allow-context-menu` attribute is not present.

```js
var DisableContextMenuExtension = MediumEditor.Extension.extend({
  name: 'disable-context-menu',

  init: function () {
    this.getEditorElements().forEach(function (element) {
      this.on(element, 'contextmenu', this.handleContextmenu.bind(this));
    }, this);
    this.subscribe('editableKeydown', this.handleKeydown.bind(this));
  },

  handleContextmenu: function (event) {
    if (!event.currentTarget.getAttribute('data-allow-context-menu')) {
      event.preventDefault();
    }
  },

  handleKeydown: function (event, editable) {
    // If the user hits escape, toggle the data-allow-context-menu attribute
    if (MediumEditor.util.isKey(event, MediumEditor.util.keyCode.ESCAPE)) {
      if (editable.hasAttribute('data-allow-context-menu')) {
        editable.removeAttribute('data-allow-context-menu');
      } else {
        editable.setAttribute('data-allow-context-menu', true);
      }
    }
  }
});
```

**NOTE**

For events like `keydown`, we could always use `currentTarget` and not need to use the reference to the editable element (like how we use the `currentTarget` when handling the `contextmenu` event).  However, there may be times when we want to trigger one of these events manually, and this allows us to specify exactly which editable element we want to trigger the event for.  It's also a handy standardization for events which are more complicated, like the custom `focus` and `blur` events.

## Extension Interface

The following are properties and method that MediumEditor will attempt to use / call to interact with the extension internally.

### `name` _(string)_

The name to identify the extension by.  This is used for calls to [`MediumEditor.getExtensionByName(name)`](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#getextensionbynamename) to retrieve the extension.  If not defined, this will be set to whatever identifier was used when passing the extension into MediumEditor via the `extensions` option.

```javascript
var MyExtension = MediumEditor.Extension.extend({
  name: 'myextension'
});

var myExt = new MyExtension();

var editor = new MediumEditor('.editor', {
  extensions: {
    'myextension': myExt
  }
});

editor.getExtensionByName(`myextension`) === myExt //true
```

***
### `init()`

Called by MediumEditor during initialization.  The `.base` property will already have been set to current instance of MediumEditor when this is called. All helper methods will exist as well.

See the [code above](#attaching-to-click) for an example of implementing the `init()` method.

***
### `checkState(node)`

If implemented, this method will be called one or more times after the state of the editor & toolbar are updated. When the state is updated, the editor does the following:

1. Find the parent node containing the current selection
2. Call `checkState(node)` on each extension, passing the node as an argument
3. Get the parent node of the previous node
4. Repeat steps #2 and #3 until we move outside the parent contenteditable

**Arguments**

1. _**node** (`Node`)_:

  * Current node, within the ancestors of the selection, that is being checked whenever a selection change occurred.

Here's an example of an extension that will add/remove a class to editor elements depending on whether or the current selection is within an element with a custom data attribute:

```javascript
var EditedExtension = MediumEditor.Extension.extend({
  name: 'edited',
  checkState: function (node) {
    // checkState is called multiple times for each selection change
    // so only store a value if the attribute was found
    if (!this.foundAttribute && node.getAttribute('data-edited')) {
      this.foundAttribute = true;
    }

    // Once we've moved up the ancestors to the container element
    // we know we're done iterating up and can add/remove the css class
    if (MediumEditor.util.isMediumEditorElement(node)) {
      if (this.foundAttribute) {
        node.classList.add('edited-text');
      } else {
        node.classList.remove('edited-text');
      }
      // Make sure the property is not persisted for the next time
      // selection is updated
      delete this.foundAttribute;
    }
  }
});

var editedExt = new EditedExtension();

var editor = new MediumEditor('.editor', {
  extensions: {
    'edited': editedExt
  }
});
```

***
### `destroy()`

If implemented, this method will be called whenever the MediumEditor is being destroyed (via a call to [`MediumEditor.destroy()](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#destroy)).

This gives the extensions the chance to remove any created html, custom event handlers or execute any other cleanup tasks that should be performed.

***
### `queryCommandState()`

If implemented, this method will be called once on each extension when the state of the editor/toolbar is being updated.

If this method returns a non-null value, the extension will be ignored as the code climbs the dom tree.

If this method returns `true`, and the `setActive()` method is defined on the extension, the `setActive()` method will be called by MediumEditor.

**Returns:** `boolean` OR `null`

***
### `isActive()`

If implemented, this method will be called from MediumEditor to determine whether the button has already been set as 'active'. 

If it returns `true`, this extension/button will be skipped for checking its active state as MediumEditor responds to the change in selection.

If it returns `false`, `isAlreadyApplied()` will still be passed each ancestor element as the MediumEditor code climbs the DOM hierarchy to respond to the change in selection.


**Returns:** `boolean`

***
### `isAlreadyApplied(node)`

If implemented, this method is similar to `checkState()` in that it will be called repeatedly as MediumEditor moves up the DOM to update the editor & toolbar after a state change.

**NOTE**: 

* This method will NOT be called if `checkState()` has been implemented. 
* This method will NOT be called if `queryCommandState()` is implemented and returns a non-null value when called.

**Arguments**

1. _**node** (`Node`)_:

  * Node to check for whether the current extension has already been applied.

**Returns:** `boolean`

***
### `setActive()`

If implemented, this method is called when MediumEditor knows that this extension is currently enabled.

Currently, this method is called when updating the editor & toolbar, and if `queryCommandState()` or `isAlreadyApplied(node)` return true when called.

***
### `setInactive()`

If implemented, this method is called when MediumEditor knows that this extension has not been applied to the current selection. Curently, this is called at the beginning of each state change for the editor & toolbar. 

After calling this, MediumEditor will attempt to update the extension, either via `checkState()` or the combination of `queryCommandState()`, `isAlreadyApplied(node)`, `isActive()`, and `setActive()`

## Extension Helpers

The following are helpers that are either set by MediumEditor during initialization, or are helper methods which either route calls to the MediumEditor instance or provide common functionality for all extensions.

### `base` _(MediumEditor)_

A reference to the instance of MediumEditor that this extension is part of.

For example, if you wanted to save the current selection within MediumEditor to be used later, you could call the following within your extension:

```javascript
this.base.saveSelection();
```

***
### `window` _(Window)_

A reference to the content window to be used by this instance of MediumEditor.  This maps to the value of the [`contentWindow`](https://github.com/yabwe/medium-editor/wiki/Options#contentwindow) option that is passed into MediumEditor.

For example, if you wanted to get the width of the window that contains this instance of MediumEditor, you could call the following within your extension:

```javascript
var windowWidth = this.window.innerWidth;
```

***
### `document` _(Document)_

A reference to the owner document to be used by this instance of MediumEditor.  This maps to the value of the [`ownerDocument`](https://github.com/yabwe/medium-editor/wiki/Options#ownerdocument) option that is passed into MediumEditor.

For example, to create an element in the current document corresponding to this instance of MediumEditor, you would call the following within your extension:

```javascript
var button = this.document.createElement('button');
```

***
### `getEditorElements()`

Returns a reference to the array of **elements** monitored by this instance of MediumEditor.

**Returns:** `Array` of `HTMLElement`s

For example, the following is the destroy method of the Placeholder Extension, which removes an attribute from all editor **elements**:

```javascript
Placeholder = Extension.extend({
  // ...
  destroy: function () {
    this.getEditorElements().forEach(function (el) {
      if (el.getAttribute('data-placeholder') === this.text) {
        el.removeAttribute('data-placeholder');
      }
    }, this);
  },
  // ...
});
```

***
### `getEditorId()`

Returns the unique identifier for this instance of MediumEditor

**Returns:** `Number`

For example, the following is an excerpt from the `createToolbar()` method of the Toolbar extension, which creates the toolbar element and gives it a unique id tied to the editor's unique id:

```javascript
Placeholder = Extension.extend({
  // ...
  createToolbar: function () {
    var toolbar = this.document.createElement('div');

    toolbar.id = 'medium-editor-toolbar-' + this.getEditorId();
    toolbar.className = 'medium-editor-toolbar';

    // ...

    return toolbar;
  },
  // ...
});
```

***
### `getEditorOption(option)`

Returns the value of a specific option used to initialize the MediumEditor object.

**Arguments**

1. _**option** ('String')_

  * Name of the MediumEditor option to retrieve.

**Returns:** Value of the MediumEditor option

For example, the following is an excerpt from the `getTemplate()` method of the Anchor extension, which checks the [`buttonLabels`](https://github.com/yabwe/medium-editor/wiki/Options#buttonlabels) option MediumEditor to decide the appearance of the 'save' button in the form:

```javascript
AnchorForm = FormExtension.extend({
  // ...
  getTemplate: function () {
    var template = [
      '<input type="text" class="medium-editor-toolbar-input" placeholder="', this.placeholderText, '">'
    ];

    template.push(
      '<a href="#" class="medium-editor-toolbar-save">',
      this.getEditorOption('buttonLabels') === 'fontawesome' ? '<i class="fa fa-check"></i>' : this.formSaveLabel,
      '</a>'
    );

    // ...
  },
  // ...
});
```

## Extension Proxy Methods

* These are methods that are just proxied calls into existing MediumEditor functions:

### `execAction(action, opts)`

Calls [`MediumEditor.execAction(action, opts)`](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#execactionaction-opts)

For example, the Button Extension will - by default - call `execAction()` each time a button is clicked, to trigger a command:

```javascript
Button = Extension.extend({
  // ...
  handleClick: function (event) {
    event.preventDefault();
    event.stopPropagation();

    var action = this.getAction(); // 'bold', 'italic', etc.

    if (action) {
      this.execAction(action);
    }
  },
  // ...
});
```

### `on(target, event, listener, useCapture)`

Calls [`MediumEditor.on(target, event, listener, useCapture)`](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#ontarget-event-listener-usecapture)

This allows extensions to easily attach event handlers to the DOM which will automatically be detached when MediumEditor is destroyed.

For example, when the Anchor Preview Extension detects a `mouseover` event for a link, it will attach to the `mouseout` event for the same link so it can hide the anchor preview:

```javascript
AnchorPreview = Extension.extend({
  // ...
  handleEditableMouseover: function (event) {
    // ...
    this.instanceHandleAnchorMouseout = this.handleAnchorMouseout.bind(this);
    this.on(this.anchorToPreview, 'mouseout', this.instanceHandleAnchorMouseout);
    // ...
  },
  // ...
});
```

### `off(target, event, listener, useCapture)`

Calls [`MediumEditor.off(target, event, listener, useCapture)`](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#offtarget-event-listener-usecapture)

To compliment the above example for `on(target, event, listener, useCapture)`, when the Anchor Preview Extension detects a `mouseout` event for a link, it will detach the the event handler for `mouseout` until the next time the mouse hovers over the link:

```javascript
AnchorPreview = Extension.extend({
  // ...
  handleAnchorMouseout: function () {
    this.anchorToPreview = null;
    this.off(this.activeAnchor, 'mouseout', this.instanceHandleAnchorMouseout);
    this.instanceHandleAnchorMouseout = null;
  },
  // ...
});
```

### `subscribe(name, listener)`

Calls [`MediumEditor.subscribe(name, listener)`](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#subscribename-listener)

For example, the Keyboard Commands Extension will subscribe to the `editableKeydown` custom event during `init()`, to monitor when keys are pressed while any of the editor **elements** are focused:

```javascript
KeyboardCommands = Extension.extend({
  // ...
  init: function () {
    Extension.prototype.init.apply(this, arguments);

    this.subscribe('editableKeydown', this.handleKeydown.bind(this));
    // ...
  },
  // ...
});
```

### `trigger(name, data, editable)`

Calls [`MediumEditor.trigger(name, data, editable)`](https://github.com/yabwe/medium-editor/wiki/MediumEditor-Object-API#triggername-data-editable)

For example, the Toolbar Extension triggers the `hideToolbar` custom event whenever the toolbar is being hidden:

```javascript
Toolbar = Extension.extend({
  // ...
  hideToolbar: function () {
    if (this.isDisplayed()) {
      this.getToolbarElement().classList.remove('medium-editor-toolbar-active');
      this.trigger('hideToolbar', {}, this.base.getFocusedElement());
    }
  },
  // ...
});
```

# Buttons

You can find a demo of this example in the source code via [button-example.html](https://github.com/yabwe/medium-editor/blob/master/demo/button-example.html).

To interact with the demo, load the page from your fork in a browser via: 

`file://[Medium Editor Source Root]/demo/button-example.html`

## Example: HighlighterButton

A simple example that uses [rangy](https://github.com/timdown/rangy) and the [CSS Class Applier Module](https://github.com/timdown/rangy/wiki/Class-Applier-Module) to support highlighting of text:

```js
rangy.init();

var HighlighterButton = MediumEditor.extensions.button.extend({
  name: 'highlighter',
  tagNames: ['mark'],
  contentDefault: '<b>H</b>',
  contentFA: '<i class="fa fa-paint-brush"></i>',
  aria: 'Hightlight',
  action: 'highlight',

  init: function () {
    MediumEditor.extensions.button.prototype.init.call(this);

    this.classApplier = rangy.createCssClassApplier('highlight', {
      elementTagName: 'mark',
      normalize: true
    });
  },

  handleClick: function (event) {
    this.classApplier.toggleSelection();
  }
});
var editor = new MediumEditor('.editable', {
  toolbar: {
    buttons: ['bold', 'italic', 'underline', 'highlighter']
  },
  buttonLabels: 'fontawesome',
  extensions: {
    'highlighter': new HighlighterButton()
  }
});
```

## Button Extension API


# Form Buttons

## Example: Anchor

## From Extension API

